@@ -65,8 +65,6 @@
 #include "config.h"
 #endif
 
-#define CLUTTER_ENABLE_EXPERIMENTAL_API
-
 #include "clutter-offscreen-effect.h"
 
 #include "cogl/cogl.h"
@@ -79,8 +77,7 @@
 struct _ClutterOffscreenEffectPrivate
 {
   CoglHandle offscreen;
-  CoglPipeline *target;
-  CoglHandle texture;
+  CoglMaterial *target;
 
   ClutterActor *actor;
   ClutterActor *stage;
@@ -88,12 +85,8 @@ struct _ClutterOffscreenEffectPrivate
   gfloat x_offset;
   gfloat y_offset;
 
-  /* This is the calculated size of the fbo before being passed
-     through create_texture(). This needs to be tracked separately so
-     that we can detect when a different size is calculated and
-     regenerate the fbo */
-  int fbo_width;
-  int fbo_height;
+  gfloat target_width;
+  gfloat target_height;
 
   gint old_opacity_override;
 
@@ -124,10 +117,10 @@ clutter_offscreen_effect_set_actor (ClutterActorMeta *meta,
   meta_class->set_actor (meta, actor);
 
   /* clear out the previous state */
-  if (priv->offscreen != NULL)
+  if (priv->offscreen != COGL_INVALID_HANDLE)
     {
       cogl_handle_unref (priv->offscreen);
-      priv->offscreen = NULL;
+      priv->offscreen = COGL_INVALID_HANDLE;
     }
 
   /* we keep a back pointer here, to avoid going through the ActorMeta */
@@ -149,6 +142,7 @@ update_fbo (ClutterEffect *effect, int fbo_width, int fbo_height)
 {
   ClutterOffscreenEffect *self = CLUTTER_OFFSCREEN_EFFECT (effect);
   ClutterOffscreenEffectPrivate *priv = self->priv;
+  CoglHandle texture;
 
   priv->stage = clutter_actor_get_stage (priv->actor);
   if (priv->stage == NULL)
@@ -160,56 +154,52 @@ update_fbo (ClutterEffect *effect, int fbo_width, int fbo_height)
       return FALSE;
     }
 
-  if (priv->fbo_width == fbo_width &&
-      priv->fbo_height == fbo_height &&
-      priv->offscreen != NULL)
+  if (priv->target_width == fbo_width &&
+      priv->target_height == fbo_height &&
+      priv->offscreen != COGL_INVALID_HANDLE)
     return TRUE;
 
-  if (priv->target == NULL)
+  if (priv->target == COGL_INVALID_HANDLE)
     {
-      CoglContext *ctx =
-        clutter_backend_get_cogl_context (clutter_get_default_backend ());
-
-      priv->target = cogl_pipeline_new (ctx);
+      priv->target = cogl_material_new ();
 
       /* We're always going to render the texture at a 1:1 texel:pixel
          ratio so we can use 'nearest' filtering to decrease the
          effects of rounding errors in the geometry calculation */
-      cogl_pipeline_set_layer_filters (priv->target,
+      cogl_material_set_layer_filters (priv->target,
                                        0, /* layer_index */
-                                       COGL_PIPELINE_FILTER_NEAREST,
-                                       COGL_PIPELINE_FILTER_NEAREST);
+                                       COGL_MATERIAL_FILTER_NEAREST,
+                                       COGL_MATERIAL_FILTER_NEAREST);
     }
 
-  if (priv->texture != NULL)
-    {
-      cogl_handle_unref (priv->texture);
-      priv->texture = NULL;
-    }
-
-  priv->texture =
+  texture =
     clutter_offscreen_effect_create_texture (self, fbo_width, fbo_height);
-  if (priv->texture == NULL)
-    return FALSE;
+  if (texture == COGL_INVALID_HANDLE)
+     return FALSE;
 
-  cogl_pipeline_set_layer_texture (priv->target, 0, priv->texture);
+  cogl_material_set_layer (priv->target, 0, texture);
+  cogl_handle_unref (texture);
 
-  priv->fbo_width = fbo_width;
-  priv->fbo_height = fbo_height;
+  /* we need to use the size of the texture target and not the minimum
+   * size we passed to the create_texture() vfunc, as any sub-class might
+   * give use a bigger texture
+   */
+  priv->target_width = cogl_texture_get_width (texture);
+  priv->target_height = cogl_texture_get_height (texture);
 
-  if (priv->offscreen != NULL)
+  if (priv->offscreen != COGL_INVALID_HANDLE)
     cogl_handle_unref (priv->offscreen);
 
-  priv->offscreen = cogl_offscreen_new_to_texture (priv->texture);
-  if (priv->offscreen == NULL)
+  priv->offscreen = cogl_offscreen_new_to_texture (texture);
+  if (priv->offscreen == COGL_INVALID_HANDLE)
     {
       g_warning ("%s: Unable to create an Offscreen buffer", G_STRLOC);
 
       cogl_handle_unref (priv->target);
-      priv->target = NULL;
+      priv->target = COGL_INVALID_HANDLE;
 
-      priv->fbo_width = 0;
-      priv->fbo_height = 0;
+      priv->target_width = 0;
+      priv->target_height = 0;
 
       return FALSE;
     }
@@ -228,7 +218,6 @@ clutter_offscreen_effect_pre_paint (ClutterEffect *effect)
   gfloat fbo_width, fbo_height;
   gfloat width, height;
   gfloat xexpand, yexpand;
-  int texture_width, texture_height;
 
   if (!clutter_actor_meta_get_enabled (CLUTTER_ACTOR_META (effect)))
     return FALSE;
@@ -259,9 +248,6 @@ clutter_offscreen_effect_pre_paint (ClutterEffect *effect)
   if (!update_fbo (effect, fbo_width, fbo_height))
     return FALSE;
 
-  texture_width = cogl_texture_get_width (priv->texture);
-  texture_height = cogl_texture_get_height (priv->texture);
-
   /* get the current modelview matrix so that we can copy it to the
    * framebuffer. We also store the matrix that was last used when we
    * updated the FBO so that we can detect when we don't need to
@@ -285,14 +271,14 @@ clutter_offscreen_effect_pre_paint (ClutterEffect *effect)
   xexpand = 0.f;
   if (priv->x_offset < 0.f)
     xexpand = -priv->x_offset;
-  if (priv->x_offset + texture_width > width)
-    xexpand = MAX (xexpand, (priv->x_offset + texture_width) - width);
+  if (priv->x_offset + priv->target_width > width)
+    xexpand = MAX (xexpand, (priv->x_offset + priv->target_width) - width);
 
   yexpand = 0.f;
   if (priv->y_offset < 0.f)
     yexpand = -priv->y_offset;
-  if (priv->y_offset + texture_height > height)
-    yexpand = MAX (yexpand, (priv->y_offset + texture_height) - height);
+  if (priv->y_offset + priv->target_height > height)
+    yexpand = MAX (yexpand, (priv->y_offset + priv->target_height) - height);
 
   /* Set the viewport */
   cogl_set_viewport (-(priv->x_offset + xexpand), -(priv->y_offset + yexpand),
@@ -347,7 +333,7 @@ clutter_offscreen_effect_real_paint_target (ClutterOffscreenEffect *effect)
 
   paint_opacity = clutter_actor_get_paint_opacity (priv->actor);
 
-  cogl_pipeline_set_color4ub (priv->target,
+  cogl_material_set_color4ub (priv->target,
                               paint_opacity,
                               paint_opacity,
                               paint_opacity,
@@ -360,8 +346,8 @@ clutter_offscreen_effect_real_paint_target (ClutterOffscreenEffect *effect)
    * hadn't been redirected offscreen.
    */
   cogl_rectangle_with_texture_coords (0, 0,
-                                      cogl_texture_get_width (priv->texture),
-                                      cogl_texture_get_height (priv->texture),
+                                      priv->target_width,
+                                      priv->target_height,
                                       0.0, 0.0,
                                       1.0, 1.0);
 }
@@ -397,8 +383,8 @@ clutter_offscreen_effect_post_paint (ClutterEffect *effect)
   ClutterOffscreenEffect *self = CLUTTER_OFFSCREEN_EFFECT (effect);
   ClutterOffscreenEffectPrivate *priv = self->priv;
 
-  if (priv->offscreen == NULL ||
-      priv->target == NULL ||
+  if (priv->offscreen == COGL_INVALID_HANDLE ||
+      priv->target == COGL_INVALID_HANDLE ||
       priv->actor == NULL)
     return;
 
@@ -449,9 +435,6 @@ clutter_offscreen_effect_finalize (GObject *gobject)
   if (priv->target)
     cogl_handle_unref (priv->target);
 
-  if (priv->texture)
-    cogl_handle_unref (priv->texture);
-
   G_OBJECT_CLASS (clutter_offscreen_effect_parent_class)->finalize (gobject);
 }
 
@@ -484,35 +467,6 @@ clutter_offscreen_effect_init (ClutterOffscreenEffect *self)
                                             ClutterOffscreenEffectPrivate);
 }
 
-/**
- * clutter_offscreen_effect_get_texture:
- * @effect: a #ClutterOffscreenEffect
- *
- * Retrieves the texture used as a render target for the offscreen
- * buffer created by @effect
- *
- * You should only use the returned texture when painting. The texture
- * may change after ClutterEffect::pre_paint is called so the effect
- * implementation should update any references to the texture after
- * chaining-up to the parent's pre_paint implementation. This can be
- * used instead of clutter_offscreen_effect_get_target() when the
- * effect subclass wants to paint using its own material.
- *
- * Return value: (transfer none): a #CoglHandle or %COGL_INVALID_HANDLE. The
- *   returned texture is owned by Clutter and it should not be
- *   modified or freed
- *
- * Since: 1.10
- */
-CoglHandle
-clutter_offscreen_effect_get_texture (ClutterOffscreenEffect *effect)
-{
-  g_return_val_if_fail (CLUTTER_IS_OFFSCREEN_EFFECT (effect),
-                        NULL);
-
-  return effect->priv->texture;
-}
-
 /**
  * clutter_offscreen_effect_get_target:
  * @effect: a #ClutterOffscreenEffect
@@ -533,9 +487,9 @@ CoglMaterial *
 clutter_offscreen_effect_get_target (ClutterOffscreenEffect *effect)
 {
   g_return_val_if_fail (CLUTTER_IS_OFFSCREEN_EFFECT (effect),
-                        NULL);
+                        COGL_INVALID_HANDLE);
 
-  return (CoglMaterial *)effect->priv->target;
+  return effect->priv->target;
 }
 
 /**
@@ -574,7 +528,7 @@ clutter_offscreen_effect_create_texture (ClutterOffscreenEffect *effect,
                                          gfloat                  height)
 {
   g_return_val_if_fail (CLUTTER_IS_OFFSCREEN_EFFECT (effect),
-                        NULL);
+                        COGL_INVALID_HANDLE);
 
   return CLUTTER_OFFSCREEN_EFFECT_GET_CLASS (effect)->create_texture (effect,
                                                                       width,
@@ -607,17 +561,17 @@ clutter_offscreen_effect_get_target_size (ClutterOffscreenEffect *effect,
   ClutterOffscreenEffectPrivate *priv;
 
   g_return_val_if_fail (CLUTTER_IS_OFFSCREEN_EFFECT (effect), FALSE);
-
+  
   priv = effect->priv;
 
-  if (priv->texture == NULL)
+  if (priv->target == NULL)
     return FALSE;
 
   if (width)
-    *width = cogl_texture_get_width (priv->texture);
+    *width = priv->target_width;
 
   if (height)
-    *height = cogl_texture_get_height (priv->texture);
+    *height = priv->target_height;
 
   return TRUE;
 }
