@@ -88,6 +88,10 @@
 #include "clutter-bezier.h"
 #include "clutter-private.h"
 
+#define CLUTTER_PATH_GET_PRIVATE(obj) \
+  (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CLUTTER_TYPE_PATH, \
+                                ClutterPathPrivate))
+
 #define CLUTTER_PATH_NODE_TYPE_IS_VALID(t) \
   ((((t) & ~CLUTTER_PATH_RELATIVE) >= CLUTTER_PATH_MOVE_TO      \
     && ((t) & ~CLUTTER_PATH_RELATIVE) <= CLUTTER_PATH_CURVE_TO) \
@@ -145,7 +149,6 @@ G_DEFINE_BOXED_TYPE (ClutterPathNode, clutter_path_node,
 G_DEFINE_TYPE_WITH_CODE (ClutterPath,
                          clutter_path,
                          G_TYPE_INITIALLY_UNOWNED,
-                         G_ADD_PRIVATE (ClutterPath)
                          CLUTTER_REGISTER_VALUE_TRANSFORM_TO (G_TYPE_STRING, clutter_value_transform_path_string)
                          CLUTTER_REGISTER_VALUE_TRANSFORM_FROM (G_TYPE_STRING, clutter_value_transform_string_path));
 
@@ -217,12 +220,14 @@ clutter_path_class_init (ClutterPathClass *klass)
                              CLUTTER_PARAM_READABLE);
   obj_props[PROP_LENGTH] = pspec;
   g_object_class_install_property (gobject_class, PROP_LENGTH, pspec);
+
+  g_type_class_add_private (klass, sizeof (ClutterPathPrivate));
 }
 
 static void
 clutter_path_init (ClutterPath *self)
 {
-  self->priv = clutter_path_get_instance_private (self);
+  self->priv = CLUTTER_PATH_GET_PRIVATE (self);
 }
 
 static void
@@ -627,14 +632,14 @@ clutter_path_parse_description (const gchar  *p,
           node = clutter_path_node_full_new ();
           nodes = g_slist_prepend (nodes, node);
 
-          node->k.type = (*p == 'M' ? CLUTTER_PATH_MOVE_TO :
-                          *p == 'm' ? CLUTTER_PATH_REL_MOVE_TO :
-                          *p == 'L' ? CLUTTER_PATH_LINE_TO :
-                          CLUTTER_PATH_REL_LINE_TO);
+          node->k.type = (*p == 'M' ? CLUTTER_PATH_MOVE_TO
+                          : *p == 'm' ? CLUTTER_PATH_REL_MOVE_TO
+                          : *p == 'L' ? CLUTTER_PATH_LINE_TO
+                          : CLUTTER_PATH_REL_LINE_TO);
           p++;
 
-          if (!clutter_path_parse_number (&p, FALSE, &node->k.points[0].x) ||
-              !clutter_path_parse_number (&p, TRUE, &node->k.points[0].y))
+          if (!clutter_path_parse_number (&p, FALSE, &node->k.points[0].x)
+              || !clutter_path_parse_number (&p, TRUE, &node->k.points[0].y))
             goto fail;
           break;
 
@@ -643,16 +648,16 @@ clutter_path_parse_description (const gchar  *p,
           node = clutter_path_node_full_new ();
           nodes = g_slist_prepend (nodes, node);
 
-          node->k.type = (*p == 'C' ? CLUTTER_PATH_CURVE_TO :
-                          CLUTTER_PATH_REL_CURVE_TO);
+          node->k.type = (*p == 'C' ? CLUTTER_PATH_CURVE_TO
+                          : CLUTTER_PATH_REL_CURVE_TO);
           p++;
 
-          if (!clutter_path_parse_number (&p, FALSE, &node->k.points[0].x) ||
-              !clutter_path_parse_number (&p, TRUE, &node->k.points[0].y) ||
-              !clutter_path_parse_number (&p, TRUE, &node->k.points[1].x) ||
-              !clutter_path_parse_number (&p, TRUE, &node->k.points[1].y) ||
-              !clutter_path_parse_number (&p, TRUE, &node->k.points[2].x) ||
-              !clutter_path_parse_number (&p, TRUE, &node->k.points[2].y))
+          if (!clutter_path_parse_number (&p, FALSE, &node->k.points[0].x)
+              || !clutter_path_parse_number (&p, TRUE, &node->k.points[0].y)
+              || !clutter_path_parse_number (&p, TRUE, &node->k.points[1].x)
+              || !clutter_path_parse_number (&p, TRUE, &node->k.points[1].y)
+              || !clutter_path_parse_number (&p, TRUE, &node->k.points[2].x)
+              || !clutter_path_parse_number (&p, TRUE, &node->k.points[2].y))
             goto fail;
           break;
 
@@ -935,7 +940,7 @@ clutter_path_get_n_nodes (ClutterPath *path)
  * clutter_path_get_node:
  * @path: a #ClutterPath
  * @index_: the node number to retrieve
- * @node: (out): a location to store a copy of the node
+ * @node: a location to store a copy of the node
  *
  * Retrieves the node of the path indexed by @index.
  *
@@ -1239,12 +1244,11 @@ clutter_path_get_description (ClutterPath *path)
   return g_string_free (str, FALSE);
 }
 
-static guint
+static gint
 clutter_path_node_distance (const ClutterKnot *start,
                             const ClutterKnot *end)
 {
-  gint64 x_d, y_d;
-  float t;
+  gint t;
 
   g_return_val_if_fail (start != NULL, 0);
   g_return_val_if_fail (end != NULL, 0);
@@ -1252,12 +1256,21 @@ clutter_path_node_distance (const ClutterKnot *start,
   if (clutter_knot_equal (start, end))
     return 0;
 
-  x_d = end->x - start->x;
-  y_d = end->y - start->y;
+  t = (end->x - start->x) * (end->x - start->x) +
+    (end->y - start->y) * (end->y - start->y);
 
-  t = floorf (sqrtf ((x_d * x_d) + (y_d * y_d)));
-
-  return (guint) t;
+  /*
+   * If we are using limited precision sqrti implementation, fallback on
+   * clib sqrt if the precission would be less than 10%
+   */
+#if INT_MAX > COGL_SQRTI_ARG_10_PERCENT
+  if (t <= COGL_SQRTI_ARG_10_PERCENT)
+    return cogl_sqrti (t);
+  else
+    return COGL_FLOAT_TO_INT (sqrtf(t));
+#else
+  return cogl_sqrti (t);
+#endif
 }
 
 static void
@@ -1371,7 +1384,7 @@ clutter_path_ensure_node_data (ClutterPath *path)
  * clutter_path_get_position:
  * @path: a #ClutterPath
  * @progress: a position along the path as a fraction of its length
- * @position: (out): location to store the position
+ * @position: location to store the position
  *
  * The value in @progress represents a position along the path where
  * 0.0 is the beginning and 1.0 is the end of the path. An
@@ -1569,65 +1582,3 @@ clutter_path_node_equal (const ClutterPathNode *node_a,
 
   return TRUE;
 }
-
-G_DEFINE_BOXED_TYPE (ClutterKnot, clutter_knot,
-                     clutter_knot_copy,
-                     clutter_knot_free);
-
-/**
- * clutter_knot_copy:
- * @knot: a #ClutterKnot
- *
- * Makes an allocated copy of a knot.
- *
- * Return value: the copied knot.
- *
- * Since: 0.2
- */
-ClutterKnot *
-clutter_knot_copy (const ClutterKnot *knot)
-{
-  if (G_UNLIKELY (knot == NULL))
-    return NULL;
-
-  return g_slice_dup (ClutterKnot, knot);
-}
-
-/**
- * clutter_knot_free:
- * @knot: a #ClutterKnot
- *
- * Frees the memory of an allocated knot.
- *
- * Since: 0.2
- */
-void
-clutter_knot_free (ClutterKnot *knot)
-{
-  if (G_LIKELY (knot != NULL))
-    g_slice_free (ClutterKnot, knot);
-}
-
-/**
- * clutter_knot_equal:
- * @knot_a: First knot
- * @knot_b: Second knot
- *
- * Compares to knot and checks if the point to the same location.
- *
- * Return value: %TRUE if the knots point to the same location.
- *
- * Since: 0.2
- */
-gboolean
-clutter_knot_equal (const ClutterKnot *knot_a,
-                    const ClutterKnot *knot_b)
-{
-  g_return_val_if_fail (knot_a != NULL, FALSE);
-  g_return_val_if_fail (knot_b != NULL, FALSE);
-
-  if (knot_a == knot_b)
-    return TRUE;
-
-  return knot_a->x == knot_b->x && knot_a->y == knot_b->y;
-}
