@@ -895,7 +895,7 @@ get_property_from_object (GObject     *gobject,
 /**
  * clutter_state_set:
  * @state: a #ClutterState instance.
- * @source_state_name: the name of the source state keys are being added for
+ * @source_state_name: (allow-none): the name of the source state keys are being added for
  * @target_state_name: the name of the target state keys are being added for
  * @first_object: a #GObject
  * @first_property_name: a property of @first_object to specify a key for
@@ -921,7 +921,7 @@ get_property_from_object (GObject     *gobject,
  *                      NULL);
  * ]|
  *
- * will create a transition from any state (a @source_state_name of NULL is
+ * will create a transition from any state (a @source_state_name or NULL is
  * treated as a wildcard) and a state named "hover"; the
  * <emphasis>button</emphasis> object will have the #ClutterActor:opacity
  * property animated to a value of 255 using %CLUTTER_LINEAR as the animation
@@ -1969,8 +1969,13 @@ clutter_state_get_duration (ClutterState *state,
   if (target_state != NULL)
     {
       if (source_state_name)
-        ret = GPOINTER_TO_INT (g_hash_table_lookup (target_state->durations,
-                                                    source_state_name));
+        {
+          ret = GPOINTER_TO_INT (g_hash_table_lookup (target_state->durations,
+                                                      source_state_name));
+          if(!ret)
+            ret = GPOINTER_TO_INT (g_hash_table_lookup (target_state->durations,
+                                                        NULL));
+        }
       else
         ret = GPOINTER_TO_INT (g_hash_table_lookup (target_state->durations,
                                                     NULL));
@@ -2136,17 +2141,17 @@ parse_state_transition (JsonArray *array,
           continue;
         }
 
-      mode = clutter_script_resolve_animation_mode (json_array_get_element (key, 2));
+      mode = _clutter_script_resolve_animation_mode (json_array_get_element (key, 2));
 
       state_key = clutter_state_key_new (target_state,
                                          gobject, property, pspec,
                                          mode);
 
-      res = clutter_script_parse_node (clos->script,
-                                       &(state_key->value),
-                                       property,
-                                       json_array_get_element (key, 3),
-                                       pspec);
+      res = _clutter_script_parse_node (clos->script,
+                                        &(state_key->value),
+                                        property,
+                                        json_array_get_element (key, 3),
+                                        pspec);
       if (!res)
         {
           g_warning ("Unable to parse the key value for the "
