@@ -246,12 +246,12 @@ clutter_event_prepare (GSource *source,
 {
   gboolean retval;
 
-  _clutter_threads_acquire_lock ();
+  clutter_threads_enter ();
 
   *timeout = -1;
   retval = (clutter_events_pending () || check_msg_pending ());
 
-  _clutter_threads_release_lock ();
+  clutter_threads_leave ();
 
   return retval;
 }
@@ -262,14 +262,14 @@ clutter_event_check (GSource *source)
   ClutterEventSource *event_source = (ClutterEventSource *) source;
   gboolean retval;
 
-  _clutter_threads_acquire_lock ();
+  clutter_threads_enter ();
 
   if ((event_source->event_poll_fd.revents & G_IO_IN))
     retval = (clutter_events_pending () || check_msg_pending ());
   else
     retval = FALSE;
 
-  _clutter_threads_release_lock ();
+  clutter_threads_leave ();
 
   return retval;
 }
@@ -282,7 +282,7 @@ clutter_event_dispatch (GSource     *source,
   ClutterEvent *event;
   MSG msg;
 
-  _clutter_threads_acquire_lock ();
+  clutter_threads_enter ();
 
   /* Process Windows messages until we've got one that translates into
      the clutter event queue */
@@ -298,7 +298,7 @@ clutter_event_dispatch (GSource     *source,
       clutter_event_free (event);
     }
 
-  _clutter_threads_release_lock ();
+  clutter_threads_leave ();
 
   return TRUE;
 }
@@ -375,9 +375,6 @@ clutter_win32_handle_event (const MSG *msg)
     return TRUE;
 
   manager = clutter_device_manager_get_default ();
-  if (manager == NULL)
-    return FALSE;
-
   core_pointer =
     clutter_device_manager_get_core_device (manager, CLUTTER_POINTER_DEVICE);
   core_keyboard =
@@ -415,24 +412,36 @@ clutter_win32_handle_event (const MSG *msg)
     case WM_ACTIVATE:
       if (msg->wParam == WA_INACTIVE)
         {
-          if (_clutter_stage_is_activated (stage_win32->wrapper))
+          if (stage_win32->state & CLUTTER_STAGE_STATE_ACTIVATED)
             {
-              _clutter_stage_update_state (stage_win32->wrapper,
-                                           CLUTTER_STAGE_STATE_ACTIVATED,
-                                           0);
+              ClutterEvent *event = clutter_event_new (CLUTTER_STAGE_STATE);
+
+              stage_win32->state &= ~CLUTTER_STAGE_STATE_ACTIVATED;
+
+              event->any.stage = stage;
+              event->stage_state.changed_mask = CLUTTER_STAGE_STATE_ACTIVATED;
+              event->stage_state.new_state = stage_win32->state;
+
+              take_and_queue_event (event);
             }
         }
-      else if (!_clutter_stage_is_activated (stage_win32->wrapper))
+      else if (!(stage_win32->state & CLUTTER_STAGE_STATE_ACTIVATED))
         {
-          _clutter_stage_update_state (stage_win32->wrapper,
-                                       0,
-                                       CLUTTER_STAGE_STATE_ACTIVATED);
+          ClutterEvent *event = clutter_event_new (CLUTTER_STAGE_STATE);
+
+          stage_win32->state |= CLUTTER_STAGE_STATE_ACTIVATED;
+
+          event->any.stage = stage;
+          event->stage_state.changed_mask = CLUTTER_STAGE_STATE_ACTIVATED;
+          event->stage_state.new_state = stage_win32->state;
+
+          take_and_queue_event (event);
         }
       break;
 
     case WM_PAINT:
-      CLUTTER_NOTE (BACKEND, "expose for stage:%p, redrawing", stage);
-      clutter_stage_ensure_redraw (stage);
+      CLUTTER_NOTE (MULTISTAGE, "expose for stage:%p, redrawing", stage);
+      clutter_redraw (stage);
       break;
 
     case WM_DESTROY:
@@ -571,7 +580,7 @@ clutter_win32_handle_event (const MSG *msg)
             clutter_event_set_device (event, core_pointer);
 
             /* we entered the stage */
-            _clutter_input_device_set_stage (core_pointer, stage);
+            _clutter_stage_add_device (stage, core_pointer);
 
             take_and_queue_event (crossing);
 
@@ -596,7 +605,7 @@ clutter_win32_handle_event (const MSG *msg)
         clutter_event_set_device (event, core_pointer);
 
         /* we left the stage */
-        _clutter_input_device_set_stage (core_pointer, NULL);
+        _clutter_stage_remove_device (stage, core_pointer);
 
         /* When we get a leave message the mouse tracking is
            automatically cancelled so we'll need to start it again when
