@@ -33,7 +33,6 @@
 #include "clutter-debug.h"
 #include "clutter-event-private.h"
 #include "clutter-main.h"
-#include "clutter-private.h"
 
 #include <string.h>
 
@@ -68,8 +67,7 @@ struct _ClutterEventSource
 {
   GSource source;
 
-  ClutterBackendX11 *backend;
-
+  ClutterBackend *backend;
   GPollFD event_poll_fd;
 };
 
@@ -102,6 +100,8 @@ static gboolean clutter_event_dispatch (GSource     *source,
                                         GSourceFunc  callback,
                                         gpointer     user_data);
 
+static GList *event_sources = NULL;
+
 static GSourceFuncs event_funcs = {
   clutter_event_prepare,
   clutter_event_check,
@@ -109,33 +109,150 @@ static GSourceFuncs event_funcs = {
   NULL
 };
 
-GSource *
-_clutter_x11_event_source_new (ClutterBackendX11 *backend_x11)
+static GSource *
+clutter_event_source_new (ClutterBackend *backend)
 {
+  GSource *source = g_source_new (&event_funcs, sizeof (ClutterEventSource));
+  ClutterEventSource *event_source = (ClutterEventSource *) source;
+
+  event_source->backend = backend;
+
+  return source;
+}
+
+static gboolean
+check_xpending (ClutterBackend *backend)
+{
+  return XPending (CLUTTER_BACKEND_X11 (backend)->xdpy);
+}
+
+#if 0
+static gboolean
+xembed_send_message (ClutterBackendX11 *backend_x11,
+                     Window             window,
+                     long               message,
+                     long               detail,
+                     long               data1,
+                     long               data2)
+{
+  XEvent ev;
+
+  memset (&ev, 0, sizeof (ev));
+
+  ev.xclient.type = ClientMessage;
+  ev.xclient.window = window;
+  ev.xclient.message_type = backend_x11->atom_XEMBED;
+  ev.xclient.format = 32;
+  ev.xclient.data.l[0] = CurrentTime;
+  ev.xclient.data.l[1] = message;
+  ev.xclient.data.l[2] = detail;
+  ev.xclient.data.l[3] = data1;
+  ev.xclient.data.l[4] = data2;
+
+  clutter_x11_trap_x_errors ();
+
+  XSendEvent (backend_x11->xdpy, window, False, NoEventMask, &ev);
+  XSync (backend_x11->xdpy, False);
+
+  if (clutter_x11_untrap_x_errors ())
+    return False;
+
+  return True;
+}
+
+static void
+xembed_set_info (ClutterBackendX11 *backend_x11,
+                 Window             window,
+                 gint               flags)
+{
+  gint32 list[2];
+
+  list[0] = MAX_SUPPORTED_XEMBED_VERSION;
+  list[1] = XEMBED_MAPPED;
+
+  XChangeProperty (backend_x11->xdpy, window,
+                   backend_x11->atom_XEMBED_INFO,
+                   backend_x11->atom_XEMBED_INFO, 32,
+                   PropModeReplace, (unsigned char *) list, 2);
+}
+#endif
+
+void
+_clutter_backend_x11_events_init (ClutterBackend *backend)
+{
+  ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
+  GSource *source;
   ClutterEventSource *event_source;
   int connection_number;
-  GSource *source;
   gchar *name;
 
   connection_number = ConnectionNumber (backend_x11->xdpy);
   CLUTTER_NOTE (EVENT, "Connection number: %d", connection_number);
 
-  source = g_source_new (&event_funcs, sizeof (ClutterEventSource));
+  source = backend_x11->event_source = clutter_event_source_new (backend);
   event_source = (ClutterEventSource *) source;
+  g_source_set_priority (source, CLUTTER_PRIORITY_EVENTS);
 
   name = g_strdup_printf ("Clutter X11 Event (connection: %d)",
                           connection_number);
   g_source_set_name (source, name);
   g_free (name);
 
-  event_source->backend = backend_x11;
   event_source->event_poll_fd.fd = connection_number;
   event_source->event_poll_fd.events = G_IO_IN;
 
+  event_sources = g_list_prepend (event_sources, event_source);
+
   g_source_add_poll (source, &event_source->event_poll_fd);
   g_source_set_can_recurse (source, TRUE);
+  g_source_attach (source, NULL);
+}
 
-  return source;
+void
+_clutter_backend_x11_events_uninit (ClutterBackend *backend)
+{
+  ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
+
+  if (backend_x11->event_source)
+    {
+      CLUTTER_NOTE (EVENT, "Destroying the event source");
+
+      event_sources = g_list_remove (event_sources,
+                                     backend_x11->event_source);
+
+      g_source_destroy (backend_x11->event_source);
+      g_source_unref (backend_x11->event_source);
+      backend_x11->event_source = NULL;
+    }
+}
+
+static void
+events_queue (ClutterBackend *backend)
+{
+  ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
+  Display *xdisplay = backend_x11->xdpy;
+  ClutterEvent *event;
+  XEvent xevent;
+
+  while (!clutter_events_pending () && XPending (xdisplay))
+    {
+      XNextEvent (xdisplay, &xevent);
+
+      event = clutter_event_new (CLUTTER_NOTHING);
+
+#ifdef HAVE_XGE
+      XGetEventData (xdisplay, &xevent.xcookie);
+#endif
+
+      if (_clutter_backend_translate_event (backend, &xevent, event))
+        _clutter_event_push (event, FALSE);
+      else
+        clutter_event_free (event);
+
+#ifdef HAVE_XGE
+      XFreeEventData (xdisplay, &xevent.xcookie);
+#endif
+    }
 }
 
 /**
@@ -183,7 +300,7 @@ clutter_x11_handle_event (XEvent *xevent)
 
   result = CLUTTER_X11_FILTER_CONTINUE;
 
-  _clutter_threads_acquire_lock ();
+  clutter_threads_enter ();
 
   backend = clutter_get_default_backend ();
 
@@ -231,7 +348,7 @@ out:
     XFreeEventData (xdisplay, &xevent->xcookie);
 #endif
 
-  _clutter_threads_release_lock ();
+  clutter_threads_leave ();
 
   return result;
 }
@@ -240,15 +357,15 @@ static gboolean
 clutter_event_prepare (GSource *source,
                        gint    *timeout)
 {
-  ClutterBackendX11 *backend = ((ClutterEventSource *) source)->backend;
+  ClutterBackend *backend = ((ClutterEventSource *) source)->backend;
   gboolean retval;
 
-  _clutter_threads_acquire_lock ();
+  clutter_threads_enter ();
 
   *timeout = -1;
-  retval = (clutter_events_pending () || XPending (backend->xdpy));
+  retval = (clutter_events_pending () || check_xpending (backend));
 
-  _clutter_threads_release_lock ();
+  clutter_threads_leave ();
 
   return retval;
 }
@@ -257,59 +374,30 @@ static gboolean
 clutter_event_check (GSource *source)
 {
   ClutterEventSource *event_source = (ClutterEventSource *) source;
-  ClutterBackendX11 *backend = event_source->backend;
+  ClutterBackend *backend = event_source->backend;
   gboolean retval;
 
-  _clutter_threads_acquire_lock ();
+  clutter_threads_enter ();
 
   if (event_source->event_poll_fd.revents & G_IO_IN)
-    retval = (clutter_events_pending () || XPending (backend->xdpy));
+    retval = (clutter_events_pending () || check_xpending (backend));
   else
     retval = FALSE;
 
-  _clutter_threads_release_lock ();
+  clutter_threads_leave ();
 
   return retval;
 }
 
-static void
-events_queue (ClutterBackendX11 *backend_x11)
-{
-  ClutterBackend *backend = CLUTTER_BACKEND (backend_x11);
-  Display *xdisplay = backend_x11->xdpy;
-  ClutterEvent *event;
-  XEvent xevent;
-
-  while (!clutter_events_pending () && XPending (xdisplay))
-    {
-      XNextEvent (xdisplay, &xevent);
-
-      event = clutter_event_new (CLUTTER_NOTHING);
-
-#ifdef HAVE_XGE
-      XGetEventData (xdisplay, &xevent.xcookie);
-#endif
-
-      if (_clutter_backend_translate_event (backend, &xevent, event))
-        _clutter_event_push (event, FALSE);
-      else
-        clutter_event_free (event);
-
-#ifdef HAVE_XGE
-      XFreeEventData (xdisplay, &xevent.xcookie);
-#endif
-    }
-}
-
 static gboolean
 clutter_event_dispatch (GSource     *source,
                         GSourceFunc  callback,
                         gpointer     user_data)
 {
-  ClutterBackendX11 *backend = ((ClutterEventSource *) source)->backend;
+  ClutterBackend *backend = ((ClutterEventSource *) source)->backend;
   ClutterEvent *event;
 
-  _clutter_threads_acquire_lock ();
+  clutter_threads_enter ();
 
   /*  Grab the event(s), translate and figure out double click.
    *  The push onto queue (stack) if valid.
@@ -325,7 +413,7 @@ clutter_event_dispatch (GSource     *source,
       clutter_event_free (event);
     }
 
-  _clutter_threads_release_lock ();
+  clutter_threads_leave ();
 
   return TRUE;
 }
@@ -375,21 +463,3 @@ clutter_x11_event_get_key_group (const ClutterEvent *event)
 
   return event_x11->key_group;
 }
-
-/**
- * clutter_x11_event_sequence_get_touch_detail:
- * @sequence: a #ClutterEventSequence
- *
- * Retrieves the touch detail froma #ClutterEventSequence.
- *
- * Return value: the touch detail
- *
- * Since: 1.12
- */
-guint
-clutter_x11_event_sequence_get_touch_detail (const ClutterEventSequence *sequence)
-{
-  g_return_val_if_fail (sequence != NULL, 0);
-
-  return GPOINTER_TO_UINT (sequence);
-}
